## Details of the plugins
#
# For now, everything defaults to being enabled - it's up to the user to specify
#    -DPLUGIN_FOO_ENABLED=FALSE
# to disable a given plugin.
#
# Defining a new plugin:
#    fast_downward_plugin(
#        NAME <NAME>
#        [ DISPLAY_NAME <DISPLAY_NAME> ]
#        [ HELP <HELP> ]
#        SOURCES
#            <FILE_1> [ <FILE_2> ... ]
#        [ DEPENDS <PLUGIN_NAME_1> [ <PLUGIN_NAME_2> ... ] ]
#        [ DEPENDENCY_ONLY ]
#        [ CORE_PLUGIN ]
#    )
#
# <DISPLAY_NAME> defaults to lower case <NAME> and is used to group
#                files in IDEs and for messages.
# <HELP> defaults to <DISPLAY_NAME> and is used to describe the cmake option.
# DEPENDS lists plugins that will be automatically enabled if this plugin
# is enabled. If the dependency was not enabled before, this will be logged.
# DEPENDENCY_ONLY disables the plugin unless it is needed as a dependency and
#     hides the option to enable the plugin in cmake GUIs like ccmake.
# CORE_PLUGIN enables the plugin and hides the option to disable it in
#     cmake GUIs like ccmake.

option(
    DISABLE_PLUGINS_BY_DEFAULT
    "If set to YES only plugins that are specifically enabled will be compiled"
    NO)
# This option should not show up in cmake GUIs like ccmake where all
# plugins are enabled or disabled manually.
mark_as_advanced(DISABLE_PLUGINS_BY_DEFAULT)

set(PLANNER_SOURCES
    planner.cc
    )

# See http://www.fast-downward.org/ForDevelopers/AddingSourceFiles
# for general information on adding source files and CMake plugins.
#
# If you're adding a file to the codebase which *isn't* a plugin, add
# it to the following list. We assume that every *.cc file has a
# corresponding *.h file and add headers to the project automatically.
# For plugin files, see below.

set(CORE_SOURCES
    abstract_task.cc
    axioms.cc
    causal_graph.cc
    cg_cache.cc
    combining_evaluator.cc
    cost_adapted_task.cc
    countdown_timer.cc
    delegating_task.cc
    deterministic_search_engine.cc
    doc.h
    domain_transition_graph.cc
    equivalence_relation.cc
    evaluation_context.cc
    evaluation_result.cc
    exact_timer.cc
    g_evaluator.cc
    global_operator.cc
    global_state.cc
    globals.cc
    heuristic.cc
    heuristic_cache.cc
    int_packer.cc
    legacy_causal_graph.cc
    operator_cost.cc
    optimization_criteria.cc
    option_parser.cc
    option_parser_util.cc
    per_state_information.cc
    plugin.h
    pref_evaluator.cc
    priority_queue.h
    prob_root_task.cc
    prob_search_engine.cc
    prob_search_node_info.cc
    prob_search_space.cc
    rng.cc
    root_task.cc
    scalar_evaluator.cc
    search_engine.cc
    search_node_info.cc
    search_progress.cc
    search_space.cc
    search_statistics.cc
    segmented_vector.cc
    state_id.cc
    state_registry.cc
    state_space.cc
    string_ops.cc
    successor_generator.cc
    sum_evaluator.cc
    system_unix.cc
    system_windows.cc
    task_proxy.cc
    task_tools.cc
    timer.cc
    tracer.cc
    utilities.cc
    utilities_hash.cc
    value_initializer.cc
    value_reporter.cc
    variable_order_finder.cc
    weighted_evaluator.cc
    )

fast_downward_add_headers_to_sources_list(CORE_SOURCES)
source_group(core FILES planner.cc ${CORE_SOURCES})
list(APPEND PLANNER_SOURCES ${CORE_SOURCES})

fast_downward_plugin(
    NAME LP_SOLVER
    SOURCES
    lp_internals.cc
    lp_solver.cc
    DEPENDENCY_ONLY)

fast_downward_plugin(
    NAME OPEN_LISTS
    SOURCES
    open_lists/open_list.cc
    open_lists/standard_scalar_open_list.cc
    open_lists/tiebreaking_open_list.cc
    open_lists/alternation_open_list.cc
    DEPENDENCY_ONLY
    )

fast_downward_plugin(
    NAME EAGER_SEARCH
    SOURCES
    eager_search.cc
    lazy_search.cc
    DEPENDS OPEN_LISTS
    )

fast_downward_plugin(
    NAME ENFORCED_HILL_CLIMBING
    SOURCES
    enforced_hill_climbing_search.cc
    DEPENDS OPEN_LISTS
    )

fast_downward_plugin(
    NAME ITERATED_SEARCH
    SOURCES
    iterated_search.cc
    DEPENDS OPEN_LISTS EAGER_SEARCH ENFORCED_HILL_CLIMBING
    )

fast_downward_plugin(
    NAME BLIND_HEURISTIC
    SOURCES
    blind_search_heuristic.cc
    )

fast_downward_plugin(
    NAME IPC_MAX_HEURISTIC
    SOURCES
    ipc_max_heuristic.cc
    )

fast_downward_plugin(
    NAME GOAL_COUNTING_HEURISTIC
    SOURCES
    goal_count_heuristic.cc
    )

fast_downward_plugin(
    NAME CG_HEURITIC
    SOURCES
    cg_heuristic.cc
    cea_heuristic.cc
    )

fast_downward_plugin(
    NAME RELAXATION_HEURISTICS
    SOURCES
    relaxation_heuristic.cc
    additive_heuristic.cc
    max_heuristic.cc
    ff_heuristic.cc
    lm_cut_heuristic.cc
    hm_heuristic.cc
    )

fast_downward_plugin(
    NAME LANDMARKS
    SOURCES
    landmarks/exploration.cc
    landmarks/h_m_landmarks.cc
    landmarks/lama_ff_synergy.cc
    landmarks/landmark_cost_assignment.cc
    landmarks/landmark_count_heuristic.cc
    landmarks/landmark_factory.cc
    landmarks/landmark_factory_rpg_exhaust.cc
    landmarks/landmark_factory_rpg_sasp.cc
    landmarks/landmark_factory_zhu_givan.cc
    landmarks/landmark_graph.cc
    landmarks/landmark_graph_merged.cc
    landmarks/landmark_status_manager.cc
    landmarks/landmark_types.h
    landmarks/util.cc
    DEPENDS LP_SOLVER
    )

fast_downward_plugin(
    NAME MAS
    SOURCES
    merge_and_shrink/budget_shrink.cc
    merge_and_shrink/distances.cc
    merge_and_shrink/factored_transition_system.cc
    merge_and_shrink/fts_factory.cc
    merge_and_shrink/heuristic_representation.cc
    merge_and_shrink/label_equivalence_relation.cc
    merge_and_shrink/labels.cc
    merge_and_shrink/merge_and_shrink_heuristic_dfs.cc
    merge_and_shrink/merge_and_shrink_heuristic.cc
    merge_and_shrink/merge_criteria.cc
    merge_and_shrink/merge_dfp.cc
    merge_and_shrink/merge_linear_extended.cc
    merge_and_shrink/merge_linear.cc
    merge_and_shrink/merge_strategy.cc
    merge_and_shrink/scc.cc
    merge_and_shrink/shrink_bisimulation.cc
    merge_and_shrink/shrink_bucket_based.cc
    merge_and_shrink/shrink_fh.cc
    merge_and_shrink/shrink_random.cc
    merge_and_shrink/shrink_strategy.cc
    merge_and_shrink/transition_system.cc
    )

fast_downward_plugin(
    NAME PDB
    SOURCES
    pdbs/canonical_pdbs_heuristic.cc
    pdbs/dominance_pruner.cc
    pdbs/match_tree.cc
    pdbs/max_cliques.cc
    pdbs/pattern_database.cc
    pdbs/pattern_generation_edelkamp.cc
    pdbs/pattern_generation_haslum.cc
    pdbs/pdb_heuristic.cc
    pdbs/util.cc
    pdbs/zero_one_pdbs_heuristic.cc
    )

fast_downward_plugin(
    NAME PROBABILISTIC
    SOURCES
    action_tie_breaking.cc
    acyclic_dfs_factory.cc
    acyclic_dfs.cc
    acyclic_value_iteration.cc
    ao_factory.cc
    aol_factory.cc
    ao_tip_list.cc
    ao_value_updater.cc
    fret_factory.cc
    fret.cc
    greedy_ao_search.cc
    lrtdp_factory.cc
    lrtdp.cc
    mas_state_space.cc
    prob_search_engine_factory.cc
    selfloop_relaxation.cc
    state_space_factory.cc
    tarjan_dfs_factory.cc
    tarjan_dfs.cc
    template_factory.cc
    topological_vi.cc
    tvi_factory.cc
    unlimited_budget_p_relaxation.cc
    value_iteration.cc
    vi_factory.cc
    DEPENDS MAS
    )

fast_downward_add_plugin_sources(PLANNER_SOURCES)

# The order in PLANNER_SOURCES influences the order in which object
# files are given to the linker, which can have a significant influence
# on performance (see issue67). The general recommendation seems to be
# to list files that define functions after files that use them.
# We approximate this by reversing the list, which will put the plugins
# first, followed by the core files, followed by the main file.
# This is certainly not optimal, but works well enough in practice.
list(REVERSE PLANNER_SOURCES)
